@@ -81,7 +81,7 @@ public:
   /**
    * Returns announce event, such as started, stopped, completed, etc.
    */
-  std::string getEventString() const;
+  const char* getEventString() const;
 
   AnnounceTier::AnnounceEvent getEvent() const;
 
@@ -130,12 +130,6 @@ public:
   bool currentTierAcceptsStoppedEvent() const;
 
   bool currentTierAcceptsCompletedEvent() const;
-
-  static const std::string STARTED;
-
-  static const std::string STOPPED;
-
-  static const std::string COMPLETED;
 };
 
 } // namespace aria2
