@@ -64,12 +64,13 @@ std::vector<std::vector<std::string> > toVector
 
 void AnnounceListTest::testSingleElementList() {
   std::string peersString = "ll8:tracker1el8:tracker2el8:tracker3ee";
-  SharedHandle<ValueBase> announcesList = bencode2::decode(peersString);
+  SharedHandle<ValueBase> announcesList =
+    bencode2::decode(peersString);
 
   // ANNOUNCE_LIST
   // [ [ tracker1 ], [ tracker2 ], [ tracker3 ] ]
   AnnounceList announceList(toVector(downcast<List>(announcesList)));
-  
+
   CPPUNIT_ASSERT(!announceList.allTiersFailed());
   std::string url =  announceList.getAnnounce();
   std::string event = announceList.getEventString();
@@ -115,7 +116,7 @@ void AnnounceListTest::testMultiElementList() {
   // ANNOUNCE_LIST
   // [ [ tracker1, tracker2, tracker3 ] ]
   AnnounceList announceList(toVector(downcast<List>(announcesList)));
-  
+
   CPPUNIT_ASSERT(!announceList.allTiersFailed());
   std::string url = announceList.getAnnounce();
   CPPUNIT_ASSERT_EQUAL(std::string("tracker1"), url);
@@ -184,13 +185,15 @@ void AnnounceListTest::testNextEventIfAfterStarted() {
   announceList.setEvent(AnnounceTier::STOPPED);
   announceList.announceFailure();
   announceList.resetTier();
-  CPPUNIT_ASSERT_EQUAL(std::string(""), announceList.getEventString());
+  CPPUNIT_ASSERT_EQUAL(std::string(""),
+                       std::string(announceList.getEventString()));
   CPPUNIT_ASSERT_EQUAL(AnnounceTier::HALTED, announceList.getEvent());
 
   announceList.setEvent(AnnounceTier::COMPLETED);
   announceList.announceFailure();
   announceList.resetTier();
-  CPPUNIT_ASSERT_EQUAL(std::string(""), announceList.getEventString());
+  CPPUNIT_ASSERT_EQUAL(std::string(""),
+                       std::string(announceList.getEventString()));
   CPPUNIT_ASSERT_EQUAL(AnnounceTier::SEEDING, announceList.getEvent());
 }
 
@@ -204,16 +207,19 @@ void AnnounceListTest::testEvent() {
 
   announceList.setEvent(AnnounceTier::STOPPED);
   announceList.announceSuccess();
-  CPPUNIT_ASSERT_EQUAL(std::string(""), announceList.getEventString());
+  CPPUNIT_ASSERT_EQUAL(std::string(""),
+                       std::string(announceList.getEventString()));
   CPPUNIT_ASSERT_EQUAL(AnnounceTier::HALTED, announceList.getEvent());
 
   announceList.setEvent(AnnounceTier::COMPLETED);
   announceList.announceSuccess();
-  CPPUNIT_ASSERT_EQUAL(std::string(""), announceList.getEventString());
+  CPPUNIT_ASSERT_EQUAL(std::string(""),
+                       std::string(announceList.getEventString()));
   CPPUNIT_ASSERT_EQUAL(AnnounceTier::SEEDING, announceList.getEvent());
 
   announceList.setEvent(AnnounceTier::STARTED_AFTER_COMPLETION);
-  CPPUNIT_ASSERT_EQUAL(std::string("started"), announceList.getEventString());
+  CPPUNIT_ASSERT_EQUAL(std::string("started"),
+                       std::string(announceList.getEventString()));
   announceList.announceSuccess();
   CPPUNIT_ASSERT_EQUAL(AnnounceTier::SEEDING, announceList.getEvent());
 }
@@ -301,7 +307,7 @@ void AnnounceListTest::testMoveToStoppedAllowedTier() {
   CPPUNIT_ASSERT_EQUAL(std::string("tracker3"), announceList.getAnnounce());
   // test wrapped search
   announceList.moveToStoppedAllowedTier();
-  CPPUNIT_ASSERT_EQUAL(std::string("tracker2"), announceList.getAnnounce());  
+  CPPUNIT_ASSERT_EQUAL(std::string("tracker2"), announceList.getAnnounce());
 }
 
 void AnnounceListTest::testMoveToCompletedAllowedTier() {
@@ -327,7 +333,7 @@ void AnnounceListTest::testMoveToCompletedAllowedTier() {
   CPPUNIT_ASSERT_EQUAL(std::string("tracker3"), announceList.getAnnounce());
   // test wrapped search
   announceList.moveToStoppedAllowedTier();
-  CPPUNIT_ASSERT_EQUAL(std::string("tracker2"), announceList.getAnnounce());  
+  CPPUNIT_ASSERT_EQUAL(std::string("tracker2"), announceList.getAnnounce());
 }
 
 } // namespace aria2
