@@ -8,6 +8,8 @@
 #include "Segment.h"
 #include "Option.h"
 #include "PieceSelector.h"
+#include "FileEntry.h"
+#include "PeerStat.h"
 
 namespace aria2 {
 
@@ -36,7 +38,7 @@ public:
     dctx_.reset
       (new DownloadContext(pieceLength, totalLength, "aria2.tar.bz2"));
     pieceStorage_.reset(new DefaultPieceStorage(dctx_, option_.get()));
-    segmentMan_.reset(new SegmentMan(option_.get(), dctx_, pieceStorage_));
+    segmentMan_.reset(new SegmentMan(dctx_, pieceStorage_));
   }
 
   void testNullBitfield();
@@ -57,22 +59,22 @@ void SegmentManTest::testNullBitfield()
   SharedHandle<DownloadContext> dctx
     (new DownloadContext(0, 0, "aria2.tar.bz2"));
   SharedHandle<UnknownLengthPieceStorage> ps
-    (new UnknownLengthPieceStorage(dctx, &op));
-  SegmentMan segmentMan(&op, dctx, ps);
+    (new UnknownLengthPieceStorage(dctx));
+  SegmentMan segmentMan(dctx, ps);
   size_t minSplitSize = dctx->getPieceLength();
 
   SharedHandle<Segment> segment = segmentMan.getSegment(1, minSplitSize);
-  CPPUNIT_ASSERT(!segment.isNull());
+  CPPUNIT_ASSERT(segment);
   CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getIndex());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getLength());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getSegmentLength());
-  CPPUNIT_ASSERT_EQUAL((size_t)0, segment->getWrittenLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getSegmentLength());
+  CPPUNIT_ASSERT_EQUAL(0, segment->getWrittenLength());
 
   SharedHandle<Segment> segment2 = segmentMan.getSegment(2, minSplitSize);
-  CPPUNIT_ASSERT(segment2.isNull());
+  CPPUNIT_ASSERT(!segment2);
 
   segmentMan.cancelSegment(1);
-  CPPUNIT_ASSERT(!segmentMan.getSegment(2, minSplitSize).isNull());
+  CPPUNIT_ASSERT(segmentMan.getSegment(2, minSplitSize));
 }
 
 void SegmentManTest::testCompleteSegment()
@@ -84,16 +86,16 @@ void SegmentManTest::testCompleteSegment()
     (new DownloadContext(pieceLength, totalLength, "aria2.tar.bz2"));
   SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, &op));
 
-  SegmentMan segmentMan(&op, dctx, ps);
+  SegmentMan segmentMan(dctx, ps);
 
-  CPPUNIT_ASSERT(!segmentMan.getSegmentWithIndex(1, 0).isNull());
+  CPPUNIT_ASSERT(segmentMan.getSegmentWithIndex(1, 0));
   SharedHandle<Segment> seg = segmentMan.getSegmentWithIndex(1, 1);
-  CPPUNIT_ASSERT(!seg.isNull());
-  CPPUNIT_ASSERT(!segmentMan.getSegmentWithIndex(1, 2).isNull());
+  CPPUNIT_ASSERT(seg);
+  CPPUNIT_ASSERT(segmentMan.getSegmentWithIndex(1, 2));
 
   seg->updateWrittenLength(pieceLength);
   segmentMan.completeSegment(1, seg);
-  
+
   std::vector<SharedHandle<Segment> > segments;
   segmentMan.getInFlightSegment(segments, 1);
   CPPUNIT_ASSERT_EQUAL((size_t)2, segments.size());
@@ -113,14 +115,14 @@ void SegmentManTest::testGetSegment_sameFileEntry()
   };
   dctx->setFileEntries(&fileEntries[0], &fileEntries[3]);
   SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, &op));
-  SegmentMan segman(&op, dctx, ps);
+  SegmentMan segman(dctx, ps);
   size_t minSplitSize =dctx->getPieceLength();
   std::vector<SharedHandle<Segment> > segments;
   segman.getSegment(segments, 1, minSplitSize, fileEntries[1], 4);
   // See 3 segments are returned, not 4 because the part of file1 is
   // not filled in segment#1
   CPPUNIT_ASSERT_EQUAL((size_t)3, segments.size());
-  
+
   SharedHandle<Segment> segmentNo1 = segman.getSegmentWithIndex(2, 1);
   // Fill the part of file1 in segment#1
   segmentNo1->updateWrittenLength(1);
@@ -148,16 +150,13 @@ void SegmentManTest::testRegisterPeerStat()
   Option op;
   SharedHandle<DownloadContext> dctx(new DownloadContext());
   SharedHandle<DefaultPieceStorage> ps(new DefaultPieceStorage(dctx, &op));
-  SegmentMan segman(&op, dctx, ps);
-  
+  SegmentMan segman(dctx, ps);
+
   SharedHandle<PeerStat> p1(new PeerStat(0, "host1", "http"));
   segman.registerPeerStat(p1);
   CPPUNIT_ASSERT_EQUAL((size_t)1, segman.getPeerStats().size());
   SharedHandle<PeerStat> p2(new PeerStat(0, "host2", "http"));
   segman.registerPeerStat(p2);
-  CPPUNIT_ASSERT_EQUAL((size_t)1, segman.getPeerStats().size());
-  p2->downloadStart();
-  segman.registerPeerStat(p1);
   CPPUNIT_ASSERT_EQUAL((size_t)2, segman.getPeerStats().size());
 }
 
@@ -165,11 +164,11 @@ void SegmentManTest::testCancelAllSegments()
 {
   segmentMan_->getSegmentWithIndex(1, 0);
   segmentMan_->getSegmentWithIndex(2, 1);
-  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(3, 0).isNull());
-  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(4, 1).isNull());
+  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(3, 0));
+  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(4, 1));
   segmentMan_->cancelAllSegments();
-  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(3, 0).isNull());
-  CPPUNIT_ASSERT(!segmentMan_->getSegmentWithIndex(4, 1).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(3, 0));
+  CPPUNIT_ASSERT(segmentMan_->getSegmentWithIndex(4, 1));
 }
 
 void SegmentManTest::testGetPeerStat()
@@ -184,17 +183,17 @@ void SegmentManTest::testGetCleanSegmentIfOwnerIsIdle()
   SharedHandle<Segment> seg1 = segmentMan_->getSegmentWithIndex(1, 0);
   SharedHandle<Segment> seg2 = segmentMan_->getSegmentWithIndex(2, 1);
   seg2->updateWrittenLength(100);
-  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(3, 0).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(3, 0));
   SharedHandle<PeerStat> peerStat3(new PeerStat(3));
   segmentMan_->registerPeerStat(peerStat3);
-  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(4, 0).isNull());
+  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(4, 0));
   SharedHandle<PeerStat> peerStat4(new PeerStat(4));
   peerStat4->downloadStart();
   segmentMan_->registerPeerStat(peerStat4);
   // Owner PeerStat is not IDLE
-  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 0).isNull());
+  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 0));
   // Segment::updateWrittenLength != 0
-  CPPUNIT_ASSERT(segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 1).isNull());
+  CPPUNIT_ASSERT(!segmentMan_->getCleanSegmentIfOwnerIsIdle(5, 1));
 }
 
 } // namespace aria2
